import QtQuick 1.0

import com.nokia.pt 1.0

import "../qtcomponents"
import "../components"

import "ApplicationModel.js" as App
import "AutoCompletionInput.js" as Logic

Item {
    id: autocompletionInput

    function isSet() {
        return isCurrentPosition || !!value;
    }

    function cancelSearch() {
        Logic.cancelSearch();
    }

    signal cleared

    property alias listArea: autocompletions.listArea
    property alias listState: autocompletions.state
    property alias delegate: autocompletions.delegate

    property variant value
    property variant preValue
    property int valueIndex: -1 // index of item selected
    property variant exclude

    property string filter // "PtStation" or null

    property bool leaveKeyboardOpen: false
    property bool isCurrentPosition: false

    property alias selected: input.selected
    property alias text: input.text
    property alias placeholderText: input.placeholderText
    property string valueText: Logic.getValueText(value, isCurrentPosition)
    property string inputText
    property alias extraMarginLeft: input.extraMarginLeft
    property int inputMarginLeft: 0

    height: input.height

    onValueChanged: {
        if (!value) {
            return;
        }
        preValue = value;

        if (value.source.indexOf("currentPosition") > -1) {
            if (!isCurrentPosition) {
                isCurrentPosition = true;
                if (App.currentPosition.notAvailable) {
                    value = undefined; // and then wait...
                } else {
                    value = Logic.copyCurrentPosition();
                }
            }
        } else {
            isCurrentPosition = false;
        }
    }

    onValueTextChanged: {
        if (!selected) {
            input.setText(valueText, true);
        }
    }

    onSelectedChanged: {
        if (selected) {
            input.setText(inputText);
            input.activateInput();
        } else {
            input.setText(valueText, true);
            //app.forceActiveFocus();
            if (!leaveKeyboardOpen) {
                input.closeSoftwareInputPanel();
            }
        }
    }

    function updateResults(newSearch) {
        Logic.updateLocationList(newSearch);
    }

    function setResults(result) {
        autocompletions.listModel = result;
    }

    onExcludeChanged: {
        if (input.text.length === 0) {
            Logic.updateLocationList();
        }
    }

    function setValue(newValue, index) {
//        if (newValue && newValue !== App.currentPosition.location) {
//            newValue.parent = input;
//        }

        if (value === newValue) {
            value = null;
        }
        valueIndex = isNaN(parseInt(index)) ? -1 : index;
        value = newValue || null;

        if (newValue) {
            App.history.add(newValue);
        }
    }

    Rectangle {
        anchors {
            fill: parent
            leftMargin: -2
            rightMargin: -2
            topMargin: -2
            bottomMargin: -2
        }
        color: "#1080DD"
        opacity: selected ? 1 : 0
        radius: 16
    }

    TextField {
        id: input

        property string inputText
        property bool extraMarginLeft: isCurrentPosition

        signal typingStopped();

        function setText(valueStr, leftAligned) {
            noTyping = true;
            text = valueStr;
            if (leftAligned) {
                cursorPosition = 0;
            }
            noTyping = false;
        }

        anchors {
            left: parent.left
            leftMargin: inputMarginLeft
            right: parent.right
        }

        Connections {
            target: App.favorites
            onAdded: {
                if (input.text.length === 0 && input.selected) {
                    autocompletionInput.updateResults();
                }
            }
        }

        onTypingStopped: {
            parent.inputText = text;
            autocompletionInput.updateResults(true);
        }

        onCleared: {
            app.forceActiveFocus();
            isCurrentPosition = false;
            value = null;
            selected = true;
            parent.inputText = "";
            autocompletionInput.updateResults();
            autocompletionInput.cleared();
        }

        MouseArea {
            z: 1000
            anchors.top: parent.top
            anchors.left: parent.left
            anchors.right: parent.right
            anchors.bottom: parent.bottom
            anchors.rightMargin: 45
            enabled: !selected

            onClicked: {
                if ((value && value.source.indexOf("currentPosition") > -1) || isCurrentPosition) {
                    isCurrentPosition = false;
                    setValue(null, -1);
                }
                selected = true;
                input.activateInput();
            }
        }

        onExtraMarginLeftChanged: {
            if (extraMarginLeft) {
                if (input.platformStyle) {
                    input.platformStyle.paddingLeft += extraMarginLeftWidth;
                } else {
                    input.platformLeftMargin += extraMarginLeftWidth;
                }
            } else {
                if (input.platformStyle) {
                    input.platformStyle.paddingLeft -= extraMarginLeftWidth;
                } else {
                    input.platformLeftMargin -= extraMarginLeftWidth;
                }
            }
        }

        onTextChanged: {
            if (!input.noTyping) {
                Logic.cancelSearch();
                input.isTyping = true;
                typingTimer.restart();
            }
        }

        onFocusChanged: {
            if (focus) {
                System.vkbStateChanged.connect(onVkbStateChanged);
                selected = true;
            } else {
                System.vkbStateChanged.disconnect(onVkbStateChanged);
            }
        }

        function onVkbStateChanged(open) {
            if (!System.splitVkb) {
                if (open) {
                    typingTimer.stop();
                    noTyping = true;
                } else {
                    isTyping = false;
                    typingStopped();
                    noTyping = false;
                }
            }
        }

        Timer {
            id: typingTimer
            interval: 350
            onTriggered: {
                if (!input.noTyping) {
                    input.isTyping = false;
                    input.typingStopped();
                }
            }
        }

        function closeSoftwareInputPanel() {
            parent.forceActiveFocus();
            //input.closeSoftwareInputPanel();
        }

        function activateInput() {
            input.selectAll();
            input.forceActiveFocus();
            input.openSoftwareInputPanel();
        }
    }

    Image {
        id: icon
        anchors.verticalCenter: parent.verticalCenter
        anchors.left: input.left
        anchors.leftMargin: 6
        source: isCurrentPosition ? "../../images/categories/position_%1.png".arg(App.currentPosition.state) : ""
    }

    Connections {
        target: isCurrentPosition ? App.currentPosition : null

        onPositionChanged: {
            if (value) {
                if (App.computeDistance(value, App.currentPosition.location) > 200) {
                    if (value.source.indexOf("currentPosition") != -1) {
                        value.title = "";
                        value.source = "";
                    }
                    isCurrentPosition = false;
                }
            } else {
                value = Logic.copyCurrentPosition();
            }
        }

        onNotAvailableChanged: {
            if (App.currentPosition.notAvailable && isCurrentPosition && value) {
                isCurrentPosition = false;
                if (value.source.indexOf("currentPosition") != -1) {
                    value.title = "";
                    value.source = "";
                }
            }
        }

        onReverseGeoCodeDone: {
            if (App.computeDistance(value, App.currentPosition.location) < 1) {
                value.address = address;
            }
        }
    }

    AutoCompletionList {
        id: autocompletions

        anchors.fill: parent
        clip: true
        input: input
        inputSelected: input.selected
        visible: false
        z: 1

        property bool selected: autocompletionInput.selected
        onSelectedChanged: {
            if (selected) {
                autocompletionInput.updateResults(false);
            } else {
                Logic.clearLocationList();
            }
        }

        onListScrolled: {
            input.closeSoftwareInputPanel();
        }

        onRetryClicked: {
            autocompletionInput.updateResults();
        }
    }
}
