import QtQuick 1.0

import com.nokia.pt 1.0

import "../../qtcomponents"
import "../../components"

import "../ApplicationModel.js" as App

Page {
    id: coveragePage
    objectName: "coveragePage"

    property variant currentPosition
    property variant currentAddress: currentPosition && currentPosition.address
    property variant coverage: currentPosition ? App.source.routingTypeAt(currentPosition) : PublicTransportRouter.ROUTING_NONE
    property string systems: currentPosition ? App.source.systemsAt(currentPosition) : ""
    property string positionName:
        (currentAddress && (currentPosition.address.city || currentPosition.address.country))
        || ""

    onStatusStrChanged: {
        if ("Activating" === statusStr) {
            app.analytics.logEvent("Coverage");
            if (App.currentPosition.notAvailable) {
                gettingPositionTimer.start();
            } else {
                currentPosition = App.currentPosition.location;
            }
        }
    }

    WelcomeHeader {
        id: header
        anchors {top: parent.top; left: parent.left; right: parent.right }
        text: qsTrId("qtn_pt_firsttimeuse_welcome");
        z: 2
    }

    Flickable {
        anchors {
            top: header.bottom
            topMargin: 10;
            bottom: coveragePage.bottom
            left: coveragePage.left
            right: coveragePage.right
        }

        contentHeight: app.inPortrait ? height : (column.height + buttons.height + 20)
        contentWidth: width

        Column {
            id: column
            anchors { left: parent.left; right: parent.right; leftMargin: 12; rightMargin: 12}
            spacing: 24

            Text {
                id: textThanksForInstalling
                anchors { left: parent.left; right: parent.right; }
                // Thanks for installing %1. Currently we support time table routing for %2 cities around the world."
                text: qsTrId("qtn_pt_coverage_thanksforinstalling").arg(System.appName).arg(System.coverage.timetableCities)
                wrapMode: Text.WrapAtWordBoundaryOrAnywhere
            }

            Item {
                id: findPositionIndicator
                anchors { left: parent.left; right: parent.right; }
                height: textGettingYourPosition.y + textGettingYourPosition.height + spinner.anchors.topMargin * 2

                BusyIndicator {
                    id: spinner
                    anchors.top: parent.top
                    anchors.topMargin: app.inPortrait ? 32 : 10
                    anchors.horizontalCenter: parent.horizontalCenter
                    visible: false
                    running: false
                    width: 64
                    height: 64
                }

                Text {
                    id: textGettingYourPosition
                    anchors { horizontalCenter: parent.horizontalCenter; top: spinner.bottom; topMargin: 8}
                }
            }

            Text {
                id: textCoverageForPosition
                anchors { left: parent.left; right: parent.right; }
                wrapMode: Text.WrapAtWordBoundaryOrAnywhere
            }
        }

        Item {
            id: buttons
            property int yOffset: 0

            anchors {
                left: parent.left;
                right: parent.right;
            }

            y: app.inPortrait ? parent.height - height : column.height + yOffset // portrait: bottom, landscape: below text
            height: skipDetectionBtn.height * 2 + 30

            Button {
                id: skipDetectionBtn

                anchors { bottom: parent.bottom; bottomMargin: 10; horizontalCenter: parent.horizontalCenter; }
                text: qsTrId("qtn_pt_coverage_button_skip") // "Skip Detection"
                visible: false
                width: 280

                onClicked: {
                    gettingPositionTimer.positioningFailed = true;
                    gettingPositionTimer.stop();
                }
            }

            Button {
                id: retryDetectionBtn

                anchors { top: parent.top; topMargin: 10; horizontalCenter: parent.horizontalCenter;  }
                text: qsTrId("qtn_pt_coverage_button_retry") // "Try again"
                visible: false
                width: 280

                onClicked: {
                    if (!gettingPositionTimer.running) {
                        gettingPositionTimer.positioningFailed = false;
                        gettingPositionTimer.start();
                    }
                }
            }

            Button {
                id: showCoveredRegionsBtn
                anchors { bottom: parent.bottom; bottomMargin: 10; horizontalCenter: parent.horizontalCenter;  }
                text: qsTrId("qtn_pt_coverage_button_showcoveredregions") // "Show covered regions"
                visible: false
                width: 280

                onClicked: {
                    leaveCoveragePage([
                        Qt.resolvedUrl("../LandingPage/LandingPage.qml"),
                        Qt.resolvedUrl("../About/SupportedRegions.qml")
                    ]);
                }
            }

            Button {
                id: continueBtn

                anchors { bottom: parent.bottom; bottomMargin: 10; horizontalCenter: parent.horizontalCenter; }
                text: qsTrId("qtn_pt_coverage_button_continue") // "Continue"
                visible: false
                width: 280
                font.bold: true

                onClicked: {
                    leaveCoveragePage("../LandingPage/LandingPage.qml");
                }
            }
        }
    }

    function leaveCoveragePage(page, params) {
        App.settings.acceptedCoverage = true;
        App.saveSettings();

        pageStack.pop(null);
        pageStack.replace(page instanceof Array ? page : Qt.resolvedUrl(page), params);
    }

    Connections {
        target: currentPosition ? null : App.currentPosition

        onNotAvailableChanged: {
            if (!App.currentPosition.notAvailable) {
                currentPosition = App.currentPosition.location;
            }
        }
    }

    Timer {
        id: gettingPositionTimer
        interval: 60000

        property bool positioningFailed: false

        onTriggered: {
            positioningFailed = true;
        }
    }

    onStateChanged: {
        if (state !== "searching for position") {
            gettingPositionTimer.stop();
        }
        console.log("FTU.state", state); // keep it for debug
    }

    states: [
        State {
            name: "position not found"
            when: gettingPositionTimer.positioningFailed && !positionName
            PropertyChanges {
                target: findPositionIndicator
                height: 0
                visible: false
            }
            PropertyChanges {
                target: textCoverageForPosition
                text: qsTrId("qtn_pt_coverage_position_noposition") // "We can't find your current position. Please check the coverage list to make sure your region is supported."
            }
            PropertyChanges {
                target: retryDetectionBtn
                visible: true
            }
            PropertyChanges {
                target: showCoveredRegionsBtn
                visible: true
            }
            PropertyChanges {
                target: continueBtn
                visible: true
            }
        },

        State {
            name: "searching for position"
            when: !positionName
            PropertyChanges {
                target: skipDetectionBtn
                visible: true
            }
            PropertyChanges {
                target: buttons
                height: skipDetectionBtn.height + 10
            }
            PropertyChanges {
                target: textGettingYourPosition
                text: qsTrId("qtn_pt_routeplanner_gps_waiting") //"Getting your position..."
            }
            PropertyChanges {
                target: spinner
                running: true
                visible: true
            }
            PropertyChanges {
                target: buttons
                yOffset: -40
            }
        },

        State {
            name: "no coverage"
            when: (currentAddress && coverage !== PublicTransportRouter.ROUTING_TIMETABLE) || false
            PropertyChanges {
                target: showCoveredRegionsBtn
                visible: true
            }
            PropertyChanges {
                target: textCoverageForPosition
                text: qsTrId("qtn_pt_coverage_position_nocoverage").arg(positionName) // "We don't have coverage for <b>%1</b>. We're adding new cities every day. Stay tuned."
            }
            PropertyChanges {
                target: header
                text: qsTrId("qtn_pt_coverage_title_nocoverage") // We're sorry
            }
            PropertyChanges {
                target: findPositionIndicator
                height: 0
                visible: false
            }
            PropertyChanges {
                target: textThanksForInstalling
                text: ""
            }
            ParentChange {
                target: textThanksForInstalling
                parent: coveragePage
            }
            PropertyChanges {
                target: showCoveredRegionsBtn
                visible: true
            }
            PropertyChanges {
                target: continueBtn
                visible: true
            }
        },

        State {
            name: "timetable coverage"
            when: (currentAddress && coverage === PublicTransportRouter.ROUTING_TIMETABLE) || false
            PropertyChanges {
                target: showCoveredRegionsBtn
                visible: true
                anchors.topMargin: 10
            }
            AnchorChanges {
                target: showCoveredRegionsBtn
                anchors.bottom: undefined
                anchors.top: buttons.top
            }

            PropertyChanges {
                target: textCoverageForPosition
                text: qsTrId("qtn_pt_coverage_position_timetablecoverage").arg(positionName).arg(systems) // "Looks like you're in <b>%1</b>.<br/>We support %2.<br/>We have covered this and many other cities."
            }
            PropertyChanges {
                target: findPositionIndicator
                height: 0
                visible: false
            }
            PropertyChanges {
                target: textThanksForInstalling
                text: ""
            }
            ParentChange {
                target: textThanksForInstalling
                parent: coveragePage
            }
            PropertyChanges {
                target: continueBtn
                visible: true
            }
        }
    ]
}
