import QtQuick 1.0

import com.nokia.pt 1.0

import "../../qtcomponents"
import "../../components"
import ".."

import "../ApplicationModel.js" as App
import "NearbyPage.js" as Logic

Page {
    id: departurePage
    objectName: "nearbyPage"

    title: qsTrId("qtn_pt_departures_title") // "Departures"

    property alias location: departureInput.value // selected location
    property alias map: mapLoader.item
    property variant departures        // available departures for stations
    property variant stations          // available stations around
    property variant stationIndex      // contails the selected station index
    property alias errorCode: errorNotification.errorCode // error will shown on the screen

    property int lateTime: 100         // how much seconds lines should shown from the past

    property variant departuresPage    // page nstance of the current departures page if active

    ListModel {
        id: stationModel
        // state: loading | loaded | error
    }

    onStatusStrChanged: {
        switch (statusStr) {
            case "Active":
                // resume open departure requests
                Logic.queryMissingStationDepartures();
                break;
            case "Deactivating":
                if (!departuresPage) {
                    App.queue.clear();
                    departureInput.cancelSearch();
                }
                break;
        }
    }

    // trigger a new station search
    onLocationChanged: {
        if (statusStr === "Active") {
            Logic.queryStations();
        }
    }

    // trigger departures search
    onStationsChanged: {
        App.queue.clear();
        if (stations) {
            departures = [];
            Logic.queryMissingStationDepartures();
        }
        if (mapLoader.ready) {
            map.departures = departures;
            map.stations = stations;
        }
    }

    // a staton is selected so the DepartureStationPage is opening
    onStationIndexChanged: {
        var index = stationIndex;
        if (undefined !== index) {
            departuresPage = App.pushPage(pageStack, Qt.resolvedUrl("NearbyStationPage.qml"), {
                station: stations[index],
                departures: departures[index]
            }, function() {
                departuresPage = undefined;
                stationIndex = undefined
            });

            if (!departures[index]) {
                App.queue.clear();
                App.queue.add(function(next, canceller) {
                    Logic.queryStationDepartures(stations[index], index, function() {
                        if (departuresPage) {
                            departuresPage.departures = departures[index];
                        }
                        next();
                    }, canceller);
                });
            }
        } else {
            if (mapLoader.ready) {
                map.stationIndex = undefined;
            }
        }
    }

    Timer {
        interval: 30000
        running: !!location && !!stations && !!departures && departurePage.visible
        repeat: true
        onTriggered: Logic.cleanup()
    }

    ViewHeading {
        id: headerBar
        objectName: "headerBar"

        anchors { top: parent.top; left: parent.left; right: parent.right }
        contentHeight: grid.height + (grid.is2ColumnLayout ? 3 : 0)

        Grid {
            id: grid
            property bool is2ColumnLayout: app.inPortrait
            anchors {
                top: parent.paddingItem.top
                left: parent.paddingItem.left
                right: parent.paddingItem.right
            }
            spacing: is2ColumnLayout ? 4 : 8
            height: is2ColumnLayout ? undefined : Math.max(textus.height, departureInput.height)
            columns: is2ColumnLayout ? 1 : 2
            Item {
                width: textus.paintedWidth
                height: grid.is2ColumnLayout ? textus.paintedHeight : parent.height
                ViewHeadingText {
                    id: textus
                    anchors {
                        left: parent.left
                        verticalCenter: parent.verticalCenter
                    }
                    item: headerBar
                    width: grid.is2ColumnLayout ? grid.width : 350
                    text: qsTrId("qtn_pt_departures_header_label")
                    wrapMode: Text.Wrap
                }
            }
            Item { // alignment container
                height: grid.is2ColumnLayout ? departureInput.height : parent.height
                width: grid.is2ColumnLayout ? parent.width : (parent.width - parent.spacing - textus.paintedWidth)
                AutoCompletionInput {
                    id: departureInput
                    anchors {
                        right: parent.right
                        verticalCenter: parent.verticalCenter
                    }
                    width: parent.width

                    value: App.currentPosition.location
                    placeholderText: ""
                    listArea: listPlaceholder
                    onValueChanged: {
                        if (value || isCurrentPosition) {
                            selected = false;
                            errorCode = QueryResult.NoError;
                            updateResults();
                        }
                    }
                    onSelectedChanged: {
                        if (selected) {
                            errorCode = QueryResult.NoError;
                        }
                    }
                }
            }
        }
    }

    Rectangle {
        id: listPlaceholder
        visible: !!departureInput.selected

        anchors.top: headerBar.bottom
        anchors.bottom: parent.bottom
        anchors.left: parent.left
        anchors.right: parent.right

        color: "#f1f1f1"
    }

    ListView {
        id: stationsList
        anchors.fill: listPlaceholder

        opacity: !!(!errorCode && !departureInput.selected && stations && buttonRow.stableCheckedButton === listButton)
                 ? 1 : 0
        Behavior on opacity { NumberAnimation { duration: 250 } }
        visible: opacity > 0

        clip: true
        model: stationModel
        delegate: NearbyStationItem {
            selected: stationIndex === index
            title: displayName
            iconSource: imageSource
            nextDepartures: displayDepartures
            enabled: isEnabled
            subItemIndicator: isMore

            onClicked: stationIndex = index;
        }
    }

    Item {
        anchors.fill: listPlaceholder

        // Loading...
        LoadingIndicator {
            visible: (location && !stations)
                   || (mapLoader.visible && !mapLoader.ready)
        }

        // Waiting for GPS...
        LoadingIndicator {
            text: qsTrId("qtn_pt_routeplanner_gps_waiting")
            visible: (departureInput.isCurrentPosition
                   && !stations
                   && !departureInput.selected
                   && App.currentPosition.notAvailable)
        }

    }

    Loader {
        id: mapLoader
        anchors.fill: listPlaceholder
        visible: !errorCode && !departureInput.selected && !!stations && buttonRow.stableCheckedButton === mapButton

        property bool ready: status === Loader.Ready

        /* MAP SIZE WORKAROUND */
        property int tmpHeight: height
        property int tmpWidth: width
        function getRoot() {
            var root = parent;
            while (root.parent) root = root.parent;
            return root;
        }
        /* /WORKAROUND */

        onReadyChanged: {
        /* MAP SIZE WORKAROUND */
            if (ready) {
                var root = getRoot();
                root.height = tmpHeight;
                root.width = tmpWidth;
                console.log("Fixed size:", root.width, root.height, tmpWidth, tmpHeight);
            }
        /* /WORKAROUND */

            if (ready && location) {
                map.location = location;
                map.departures = departures;
                map.stations = stations;
                map.stationIndexChanged.connect(function() {
                    stationIndex = map.stationIndex;
                });
            }
        }
    }

    StationErrorNotification {
        id: errorNotification
        errorCode: errorCode

        onRetryClicked: Logic.queryStations();
    }

    tools: ToolBarLayout {

        BackToolButton {
            objectName: "backBtn"
        }

        ButtonRow {
            id: buttonRow
            exclusive: true
            visible: System.pluginPresent
            property variant stableCheckedButton: listButton

            ToolButton {
                id: listButton
                iconSource: "../../../images/toolbar/list.png"
                checkable: true
                onClicked: {
                    buttonRow.stableCheckedButton = listButton;
                }
            }

            ToolButton {
                id: mapButton
                iconSource: "../../../images/toolbar/map.png"
                checkable: true
                onClicked: {
                    buttonRow.stableCheckedButton = mapButton;
                    if (mapLoader.source !== "DepartureMap.qml") {
                        /* MAP SIZE WORKAROUND */
                        var root = mapLoader.getRoot();
                        mapLoader.tmpWidth = root.width;
                        mapLoader.tmpHeight = root.height;
                        /* /WORKAROUND */
                        mapLoader.source = "DepartureMap.qml";
                    }
                }
            }
            onCheckedButtonChanged: {
                checkedButton = stableCheckedButton; // workaround
            }
        }
    }

    // workaround to get all changes. better to use onVisible events but they fired twice (i don't know why)
    property variant analyticsStatus: "Active" === departurePage.statusStr && buttonRow.stableCheckedButton
    onAnalyticsStatusChanged: {
        if (analyticsStatus) {
            if (listButton === buttonRow.stableCheckedButton) {
                app.analytics.logEvent("#StationsAsList");
            } else if (mapButton === buttonRow.stableCheckedButton) {
                app.analytics.logEvent("#StationsOnMap");
            }
        }
    }
}
