import QtQuick 1.0

import com.nokia.pt 1.0

import "../../components"
import "../../components/styles"

import "../ApplicationModel.js" as App

ListItem {
    id: calendarEntry

    property variant model
    property real timeScale: 600000
    property int timeSliceWidth: 75

    property int contentX: 0
    property int contentWidth: 0

    property real currentTime: 0
    property int currentTimeOffset: 0

    property int xOffset: 0
    property int connectionLength: Math.floor(model.duration/timeScale * timeSliceWidth)

    style: ListItemStyle {
        color: "#6d6d6d"
    }

    width: contentWidth
    height: 110

    function buildStartTimeString(currTime, tripStart) {
        var minutes = Math.ceil((tripStart - currTime)/60000);
        if (minutes > 120) {
            return System.formatDateTime(tripStart, "time").datetime;
        }
        return qsTrId("qtn_pt_journeycalendaritem_in_minutes").arg(minutes);
    }

    onModelChanged: {
        var getWidth = function(time1, time2) {
            return Math.ceil(Math.abs(time2 - time1)/timeScale * timeSliceWidth);
        };

        var blocks = [];
        var totalDuration = 0;

        if (model && model.steps.length) {
            var lastTime = model.startTime;
            for (var i = 0, l = model.steps.length; i < l; i++) {
                var step = model.steps[i];
                var width;

                if (step.timetableBased) {
                    // add waiting time
                    if ((step.startTime - lastTime) > 60000) {
                        width = getWidth(lastTime, step.startTime);
                        lastTime = step.startTime;
                        blocks.push({
                            width: width,
                            blurStart: 0,
                            blurEnd: 0,
                            blurred: false,
                            color: "#ccc",
                            textColor: "#000",
                            icon: step.timetableBased ? "../../../images/journeyplanner/wait_white.png" : "",
                            text: "",
                            departureDiff: 0,
                            hasDelayInfo: false,
                            seperator: 0
                        });
                        totalDuration += step.startTime - lastTime;
                    }
                    width = getWidth(lastTime, step.endTime);
                    lastTime = step.endTime;
                    totalDuration += step.endTime - step.startTime;
                    blocks.push({
                        width: width,
                        blurStart: 0,
                        blurEnd: 0,
                        blurred: false,
                        color: step.line.lineScheme.color,
                        textColor: step.line.lineScheme.textColor || "#FFF",
                        icon: step.line.isWalk ? "../../../images/journeyplanner/walk_white.png" : "",
                        text: step.line.name || "",
                        departureDiff: step.departureDiff / 60000,
                        hasDelayInfo: step.hasDelayInfo,
                        seperator: 0
                    });
                } else {
                    lastTime = step.endTime;
                    blocks.push({
                        width: getWidth(step.startTime, step.endTime),
                        blurStart: getWidth(0, -step.departureDiff),
                        blurEnd: getWidth(0, step.arrivalDiff),
                        blurred: true,
                        color: step.line.lineScheme.color,
                        textColor: step.line.lineScheme.textColor || "#FFF",
                        icon: step.line.isWalk ? "../../../images/journeyplanner/walk_white.png" : "",
                        text: step.line.name || "",
                        hasDelayInfo: step.hasDelayInfo,
                        seperator: 0
                    });
                }

                // Have border when same color twice
                if (blocks.length > 1 && blocks[blocks.length-1].color == blocks[blocks.length-2].color) {
                    blocks[blocks.length-1].seperator = 1;
                }
            }
        }

        stepper.model = blocks;
    }

    Item {
        id: content
        x: xOffset
        width: connectionLength
        anchors.top: parent.top
        anchors.bottom: parent.bottom

        Row {
            id: lineRects
            anchors.verticalCenter: parent.verticalCenter
            height: 40
            spacing: 1

            Repeater {
                id: stepper

                delegate: Rectangle {
                    id: stepItem
                    anchors.top: parent.top
                    anchors.bottom: parent.bottom
                    width: modelData.width + modelData.blurStart + modelData.blurEnd

                    property bool blurred: modelData.blurStart + modelData.blurEnd > 1

                    LinearGradientRectangle {
                        id: startGrad
                        anchors { left: parent.left; top: parent.top; bottom: parent.bottom; }
                        visible: width > 0
                        width: (stepItem.blurred && modelData.blurStart > 1) ? modelData.blurStart : 0
                        direction: LinearGradientRectangle.LEFT_TO_RIGHT
                        property color gradientColor: modelData.color ? "#54" + ("" + Qt.tint(modelData.color, modelData.color)).substr(1, 6) : "transparent"

                        startColor: "transparent"
                        endColor: startGrad.gradientColor || "transparent"
                    }

                    LinearGradientRectangle {
                        anchors { left: stepRect.right; leftMargin: 1; top: parent.top; bottom: parent.bottom; right: parent.right; }
                        visible: stepItem.blurred
                        direction: LinearGradientRectangle.LEFT_TO_RIGHT

                        startColor: startGrad.gradientColor || "transparent"
                        endColor: "transparent"
                    }

                    Rectangle {
                        id: stepRect
                        anchors {
                            left: startGrad.right
                            leftMargin: stepItem.blurred ? 1 : modelData.seperator
                            top: parent.top;
                            bottom: parent.bottom
                        }
                        color: modelData.color
                        width: modelData.width
                        clip: true

                        Image {
                            id: icon
                            source: modelData.icon
                            asynchronous: true
                            anchors.centerIn: parent
                            clip: true
                            visible: modelData.icon !== ""
                            fillMode: Image.PreserveAspectFit
                            height: Math.min(parent.height, sourceSize.height)
                            width: Math.min(parent.width, sourceSize.width)
                            smooth: true
                        }
                        Item {}

                        Text {
                            id: lineText
                            color: modelData.textColor
                            text: modelData.text
                            smooth:true

                            anchors.centerIn: parent

                            font.pixelSize: 17
                            font.family: "Nokia Sans TitleSmBd S60"
                            clip: true
                            Item {
                                id: ellideGradient
                                anchors.fill: parent
                                visible: false // enabled by clipped state
                                Rectangle {
                                    anchors.right: parent.right
                                    width:6
                                    anchors.top: parent.top
                                    anchors.bottom: parent.bottom
                                    color:modelData.color
                                    opacity: 0.5

                                }
                                Rectangle {
                                    anchors.right: parent.right
                                    width:4
                                    anchors.top: parent.top
                                    anchors.bottom: parent.bottom
                                    color:modelData.color
                                    opacity: 0.5
                                    visible: lineText.paintedWidth >= modelData.width - 3
                                }
                                Rectangle {
                                    anchors.right: parent.right
                                    width:2
                                    anchors.top: parent.top
                                    anchors.bottom: parent.bottom
                                    color:modelData.color
                                    opacity: 0.7
                                    visible: lineText.paintedWidth >= modelData.width - 3
                                }
                            }

                            states: [
                                State {
                                name: "clipped"
                                when: parent !== undefined && lineText.paintedWidth >= modelData.width - 3

                                PropertyChanges {
                                    target: lineText
                                    width: stepRect.width - 3
                                    anchors.leftMargin: 2
                                }
                                PropertyChanges {
                                    target: ellideGradient
                                    visible: true
                                }
                            }
                            ]

                        }
                    }

                    Rectangle {
                        anchors { left: stepRect.left; leftMargin: -1; top: stepRect.top; bottom: stepRect.bottom; }
                        color: modelData.color
                        opacity: 0.8
                        width: modelData.seperator
                    }
                }
            }
        }

        // journey start time
        ListItemText {
            id: startTime
            item: calendarEntry
            opacity: (currentTimeOffset < xOffset) &&
                     (startTime.x + startTime.width + 15 < duration.x) ? 1 : 0
            text: buildStartTimeString(currentTime, model.startTime)

            anchors {left: lineRects.left; bottom: lineRects.top; bottomMargin: 1}
            font    {pixelSize: 18; bold: false; letterSpacing: -1}

            Behavior on opacity {
                 NumberAnimation { properties:"opacity"; duration: 200 }
            }
        }

        // journey duration
        DateTime {
            id: duration
            datetime: model.timetableBased ? model.duration : model.estimatedDuration
            format: "duration"
            estimate: !model.timetableBased

            x: 0
            anchors {bottom: lineRects.top; right: lineRects.right; topMargin: -1}
            font: startTime.font
            color: startTime.color
        }

        // journey arrival time
        DateTime {
            id: arrivalTime
            visible: model.timetableBased
            datetime: model.startTime + model.duration

            anchors {top: lineRects.bottom; right: lineRects.right; topMargin: 2}
            font: startTime.font
            color: startTime.color
        }
    }

    Image {
        property int direction: (calendarEntry.x + calendarEntry.width < flickie.contentX) ? -1 :
                (calendarEntry.x + xOffset >  flickie.contentX + flickie.width) ? 1 : 0
        anchors.verticalCenter: parent.verticalCenter
        x: flickie.contentX + ((direction < 0) ?  + 7 : flickie.width - 25)
        width: 18
        height: 32
        asynchronous: true
        source: "../../../images/" + ((direction  < 0 ) ? "earlier" : "later") + "_connection_indicator.png"
        visible: direction != 0
    }
}
