import QtQuick 1.0

import com.nokia.pt 1.0

import "../qtcomponents"
import "../components"
import "../components/styles/Styles.js" as Styles
import "ApplicationModel.js" as App
import "Analytics" 1.0
import "models"
import "FirstTimeUse"
import "Ticketing"

PageStackWindow {
    id: app

    property bool portrait: inPortrait
    property Analytics analytics: Analytics {}

    signal sourceCompleted
    property bool startupCompleted: false

    PublicTransportRouter {
        id: source
    }

    Component.onCompleted: App.init(app, "native", source, System);

    onSourceCompleted: {
        try {
            App.sourceCompleted(app);
            Styles.initialize(app)
            if (!App.settings.acceptedTnC) {
                pageStack.push(termsAndConditions);
            } else if (!App.settings.acceptedCoverage) {
                pageStack.push(Qt.resolvedUrl("FirstTimeUse/CoveragePage.qml"));
            } else {
                if (PtApplication.argTo) {
                    console.log("Arguments: Called with destination so going directly to JourneyPlanner.");
                    pageStack.push(Qt.resolvedUrl("RoutePlanner/RoutePlannerPage.qml"), {
                        toValue: PtApplication.argTo
                    });
                } else if (PtApplication.argNearby) {
                    console.log("Arguments: Called with nearby so going directly to NearbyPage.");
                    pageStack.push(Qt.resolvedUrl("Nearby/NearbyPage.qml"), {
                        location: PtApplication.argNearby
                    });
                } else {
                    pageStack.push(Qt.resolvedUrl("LandingPage/LandingPage.qml"));
                }
            }
        } catch(e) {
            errorText.text = e.message;
            console.log(e.message);
        }

        startupCompleted = true;

        Rumble.success();

        Ticket.activate();

        // log first getting position
        var firstPosition = function() {
            App.currentPosition.positionChanged.disconnect(firstPosition);
            app.analytics.logEvent("GotFirstPosition", {
                    appTime: Stats.uptime(),
                    gpsMethod: App.currentPosition.method || "-",
                    accuracy: App.currentPosition.accuracy
                });
        }
        App.currentPosition.positionChanged.connect(firstPosition);

        startUpTimer.start();
    }

    // execute it in different event loop because of S^1
    Timer {
        id: startUpTimer
        interval: 1
        onTriggered: {
            app.analytics.logEvent("ApplicationStart", {
                    appTime: Stats.uptime(),
                    locale: Stats.locale(),
                    totalFavorites: (App.favorites.get(false) || []).length,
                    stickyFavorites: (App.favorites.get(true) || []).length
                });
        }
    }

    Component {
        id: termsAndConditions
        TermsPage {
            onAccepted: {
                App.settings.acceptedTnC = true;
                App.saveSettings();
                pageStack.replace(Qt.resolvedUrl("FirstTimeUse/CoveragePage.qml"));
            }
            onDeclined: {
                app.analytics.logEvent("DeclinedTermsAndConditions");
                app.quit();
            }
        }
    }

    // The red error message if the startup failed
    Rectangle {
        color: "#cc3333"
        anchors.fill: parent
        z: 10000
        visible: errorText.text.length > 0

        Label {
            id: errorText
            width: parent.width
            text: ""
            font.pixelSize: 18
            color: "#fff"
            wrapMode: "WrapAtWordBoundaryOrAnywhere"
        }

        MouseArea {
            anchors.fill: parent
            onClicked: app.quit()
        }
    }

    // Ticketing connector
    Loader {
        Component {
            id: ticketDialogConnector
            DialogConnector { // todo: refactor to get not loading this class
                parent: app
            }
        }
        sourceComponent: Ticket.available ? ticketDialogConnector : undefined
    }
    // -------------------

    // Toast messy should be the last element to prevent z-usage
    property ToastMessage toast: ToastMessage {
        parent: app
        anchors.topMargin: 30
        onErrorMessage: console.error("Toast:", message);
        onSuccessMessage: console.log("Toast:", message);
        onCustomMessage: console.log("Toast:", message, type ? "(" + type+ ")" : undefined);
    }

    function displayMessage(type, description, errorCode) {
        if (type === "error") {
            app.analytics.logEvent("Error", errorCode);
            if (description.indexOf("%1") !== -1) {
                description = description.arg(getErrorText(errorCode));
            }
        }

        description = description || "Something bad happened";
        switch (type) {
            case "error": toast.error(description);
                break;
            case "info": toast.info(description);
                break;
            default: toast.custom(description);
        }
    }

    function getErrorText(errorCode) {
        switch (errorCode) {
        case QueryResult.NoError:
            return "";
        case QueryResult.NoResults:
            return qsTrId("qtn_pt_error_noresults");//.arg(errorCode);
        case QueryResult.OtherError:
            return "Error";
        case QueryResult.Timeout:
        case QueryResult.BadData:
        case QueryResult.BadInput:
            return qsTrId("qtn_pt_error_network_release");
        case QueryResult.NotImplemented:
            return "Not implemented.";
        case QueryResult.NoDataAvailable:
            return qsTrId("qtn_pt_error_area_not_supported");
        }

        if (errorCode & QueryResult.NetworkError) {
            return qsTrId("qtn_pt_error_network_release");
        }

        return "Error";
    }

    function quit() {
        var fps = Stats.fps();
        app.analytics.logEvent("ClosingApp", {
                appTime: Stats.uptime(),
                numRequests: Stats.netNumRequests(),
                numResponsesSuccess: Stats.netNumSucceededResponses(),
                numResponsesError: Stats.netNumFailedResponses(),
                overallAvgResponseDuration: Stats.netAvgDuration(),
                overallSentBytes: Stats.netBytesSent(),
                overallReceivedBytes: Stats.netBytesReceived(),
                fps0: fps[0],
                fps10: fps[1],
                fps20: fps[2],
                fps30: fps[3],
                fps40: fps[4],
                fps50: fps[5],
                fps60: fps[6]
            });
        app.analytics.closeSession();
        Qt.quit();
    }
}
